@@ -8,8 +8,8 @@
 
 void _shell_global_set_plugin      (ShellGlobal  *global,
                                     MetaPlugin   *plugin);
-void _shell_global_set_gjs_context (ShellGlobal  *global,
-                                    GjsContext   *context);
+
+GjsContext *_shell_global_get_gjs_context (ShellGlobal  *global);
 
 gboolean _shell_global_check_xdnd_event (ShellGlobal  *global,
                                          XEvent       *xev);
