@@ -9,6 +9,10 @@
 #include <X11/Xatom.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
+#include <meta/display.h>
+#include <meta/group.h>
+#include <meta/util.h>
+#include <meta/window.h>
 
 #define SN_API_NOT_YET_FROZEN 1
 #include <libsn/sn.h>
@@ -20,11 +24,6 @@
 #include "shell-marshal.h"
 #include "st.h"
 
-#include "display.h"
-#include "window.h"
-#include "group.h"
-#include "util.h"
-
 /* This file includes modified code from
  * desktop-data-engine/engine-dbus/hippo-application-monitor.c
  * in the functions collecting application usage data.
@@ -226,23 +225,6 @@ get_appid_from_window (MetaWindow  *window)
   return appid_guess;
 }
 
-/**
- * window_is_tracked:
- *
- * We don't attempt to associate override-redirect windows with applications
- * at all, since there's no reason to do so yet.
- *
- * Returns: %TRUE iff we want to scan this window for application association
- */
-static gboolean
-window_is_tracked (MetaWindow *window)
-{
-  if (meta_window_is_override_redirect (window))
-    return FALSE;
-
-  return TRUE;
-}
-
 /**
  * shell_window_tracker_is_window_interesting:
  *
@@ -530,7 +512,7 @@ track_window (ShellWindowTracker *self,
 {
   ShellApp *app;
 
-  if (!window_is_tracked (window))
+  if (!shell_window_tracker_is_window_interesting (window))
     return;
 
   app = get_app_for_window (self, window);
@@ -540,10 +522,6 @@ track_window (ShellWindowTracker *self,
   /* At this point we've stored the association from window -> application */
   g_hash_table_insert (self->window_to_app, window, app);
 
-  /* However, only put interesting windows in the window list for an app. */
-  if (!shell_window_tracker_is_window_interesting (window))
-    return;
-
   if (shell_app_is_transient (app))
     {
       /* For a transient application, it's possible one of our title regexps
