@@ -68,17 +68,39 @@ struct _StButtonClass
 
 GType st_button_get_type (void) G_GNUC_CONST;
 
-StWidget *            st_button_new             (void);
-StWidget *            st_button_new_with_label  (const gchar *text);
-G_CONST_RETURN gchar *st_button_get_label       (StButton    *button);
-void                  st_button_set_label       (StButton    *button,
-                                                 const gchar *text);
-void                  st_button_set_toggle_mode (StButton    *button,
-                                                 gboolean     toggle);
-gboolean              st_button_get_toggle_mode (StButton    *button);
-void                  st_button_set_checked     (StButton    *button,
-                                                 gboolean     checked);
-gboolean              st_button_get_checked     (StButton    *button);
+StWidget    *st_button_new             (void);
+StWidget    *st_button_new_with_label  (const gchar  *text);
+const gchar *st_button_get_label       (StButton     *button);
+void         st_button_set_label       (StButton     *button,
+                                        const gchar  *text);
+void         st_button_set_toggle_mode (StButton     *button,
+                                        gboolean      toggle);
+gboolean     st_button_get_toggle_mode (StButton     *button);
+void         st_button_set_checked     (StButton     *button,
+                                        gboolean      checked);
+gboolean     st_button_get_checked     (StButton     *button);
+
+void         st_button_fake_release    (StButton     *button);
+
+/**
+ * StButtonMask:
+ * @ST_BUTTON_ONE: button 1 (left)
+ * @ST_BUTTON_TWO: button 2 (middle)
+ * @ST_BUTTON_THREE: button 3 (right)
+ *
+ * A mask representing which mouse buttons an StButton responds to.
+ */
+typedef enum {
+  ST_BUTTON_ONE   = (1 << 0),
+  ST_BUTTON_TWO   = (1 << 1),
+  ST_BUTTON_THREE = (1 << 2),
+} StButtonMask;
+
+#define ST_BUTTON_MASK_FROM_BUTTON(button) (1 << ((button) - 1))
+
+void         st_button_set_button_mask (StButton     *button,
+                                        StButtonMask  mask);
+StButtonMask st_button_get_button_mask (StButton     *button);
 
 G_END_DECLS
 
