@@ -4,6 +4,10 @@
  *
  * Copyright 2007 OpenedHand
  * Copyright 2008, 2009 Intel Corporation.
+ * Copyright 2009, 2010 Red Hat, Inc.
+ * Copyright 2009 Abderrahim Kitouni
+ * Copyright 2009, 2010 Florian Müllner
+ * Copyright 2010 Adel Gadllah
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -15,13 +19,7 @@
  * more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
- * Boston, MA 02111-1307, USA.
- *
- * Written by: Emmanuele Bassi <ebassi@openedhand.com>
- *             Thomas Wood <thomas@linux.intel.com>
- *
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,6 +42,10 @@
 #include "st-tooltip.h"
 #include "st-theme-node-transition.h"
 
+#include "st-widget-accessible.h"
+
+#include <gtk/gtk.h>
+
 /*
  * Forward declaration for sake of StWidgetChild
  */
@@ -56,18 +58,23 @@ struct _StWidgetPrivate
   gchar        *inline_style;
 
   StThemeNodeTransition *transition_animation;
+  guint tooltip_timeout_id;
 
   gboolean      is_stylable : 1;
   gboolean      has_tooltip : 1;
+  gboolean      show_tooltip : 1;
   gboolean      is_style_dirty : 1;
   gboolean      draw_bg_color : 1;
   gboolean      draw_border_internal : 1;
   gboolean      track_hover : 1;
   gboolean      hover : 1;
+  gboolean      can_focus : 1;
 
   StTooltip    *tooltip;
 
   StTextDirection   direction;
+
+  AtkObject *accessible;
 };
 
 /**
@@ -93,12 +100,14 @@ enum
   PROP_HAS_TOOLTIP,
   PROP_TOOLTIP_TEXT,
   PROP_TRACK_HOVER,
-  PROP_HOVER
+  PROP_HOVER,
+  PROP_CAN_FOCUS
 };
 
 enum
 {
   STYLE_CHANGED,
+  POPUP_MENU,
 
   LAST_SIGNAL
 };
@@ -113,6 +122,15 @@ G_DEFINE_ABSTRACT_TYPE (StWidget, st_widget, CLUTTER_TYPE_ACTOR);
 
 static void st_widget_recompute_style (StWidget    *widget,
                                        StThemeNode *old_theme_node);
+static gboolean st_widget_real_navigate_focus (StWidget         *widget,
+                                               ClutterActor     *from,
+                                               GtkDirectionType  direction);
+
+static AtkObject * st_widget_get_accessible (ClutterActor *actor);
+
+static void st_widget_start_tooltip_timeout (StWidget *widget);
+static void st_widget_do_show_tooltip (StWidget *widget);
+static void st_widget_do_hide_tooltip (StWidget *widget);
 
 static void
 st_widget_set_property (GObject      *gobject,
@@ -164,6 +182,10 @@ st_widget_set_property (GObject      *gobject,
       st_widget_set_hover (actor, g_value_get_boolean (value));
       break;
 
+    case PROP_CAN_FOCUS:
+      st_widget_set_can_focus (actor, g_value_get_boolean (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -217,6 +239,10 @@ st_widget_get_property (GObject    *gobject,
       g_value_set_boolean (value, priv->hover);
       break;
 
+    case PROP_CAN_FOCUS:
+      g_value_set_boolean (value, priv->can_focus);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -255,21 +281,25 @@ st_widget_dispose (GObject *gobject)
 
   st_widget_remove_transition (actor);
 
+  if (priv->tooltip_timeout_id)
+    {
+      g_source_remove (priv->tooltip_timeout_id);
+      priv->tooltip_timeout_id = 0;
+    }
+
   if (priv->tooltip)
     {
-      ClutterContainer *parent;
-      ClutterActor *tooltip = CLUTTER_ACTOR (priv->tooltip);
-
-      /* this is just a little bit awkward because the tooltip is parented
-       * on the stage, but we still want to "own" it */
-      parent = CLUTTER_CONTAINER (clutter_actor_get_parent (tooltip));
-
-      if (parent)
-        clutter_container_remove_actor (parent, tooltip);
-
+      clutter_actor_destroy (CLUTTER_ACTOR (priv->tooltip));
+      g_object_unref (priv->tooltip);
       priv->tooltip = NULL;
     }
 
+  /* The real dispose of this accessible is done on
+   * AtkGObjectAccessible weak ref callback
+   */
+  if (priv->accessible)
+    priv->accessible = NULL;
+
   G_OBJECT_CLASS (st_widget_parent_class)->dispose (gobject);
 }
 
@@ -334,13 +364,10 @@ st_widget_allocate (ClutterActor          *actor,
 {
   StWidget *self = ST_WIDGET (actor);
   StWidgetPrivate *priv = self->priv;
-  StThemeNode *theme_node;
   ClutterActorClass *klass;
   ClutterGeometry area;
   ClutterVertex in_v, out_v;
 
-  theme_node = st_widget_get_theme_node (self);
-
   klass = CLUTTER_ACTOR_CLASS (st_widget_parent_class);
   klass->allocate (actor, box, flags);
 
@@ -405,25 +432,28 @@ st_widget_parent_set (ClutterActor *widget,
 static void
 st_widget_map (ClutterActor *actor)
 {
-  StWidgetPrivate *priv = ST_WIDGET (actor)->priv;
+  StWidget *self = ST_WIDGET (actor);
 
   CLUTTER_ACTOR_CLASS (st_widget_parent_class)->map (actor);
 
-  st_widget_ensure_style ((StWidget*) actor);
+  st_widget_ensure_style (self);
 
-  if (priv->tooltip)
-    clutter_actor_map ((ClutterActor *) priv->tooltip);
+  if (self->priv->show_tooltip)
+    st_widget_do_show_tooltip (self);
 }
 
 static void
 st_widget_unmap (ClutterActor *actor)
 {
-  StWidgetPrivate *priv = ST_WIDGET (actor)->priv;
+  StWidget *self = ST_WIDGET (actor);
+  StWidgetPrivate *priv = self->priv;
 
   CLUTTER_ACTOR_CLASS (st_widget_parent_class)->unmap (actor);
 
-  if (priv->tooltip)
-    clutter_actor_unmap ((ClutterActor *) priv->tooltip);
+  if (priv->track_hover && priv->hover)
+    st_widget_set_hover (self, FALSE);
+
+  st_widget_do_hide_tooltip (self);
 }
 
 static void notify_children_of_style_change (ClutterContainer *container);
@@ -512,6 +542,9 @@ get_root_theme_node (ClutterStage *stage)
  * The theme node is used to access standard and custom CSS
  * properties of the widget.
  *
+ * Note: it is a fatal error to call this on a widget that is
+ *  not been added to a stage.
+ *
  * Return value: (transfer none): the theme node for the widget.
  *   This is owned by the widget. When attributes of the widget
  *   or the environment that affect the styling change (for example
@@ -528,6 +561,7 @@ st_widget_get_theme_node (StWidget *widget)
       StThemeNode *parent_node = NULL;
       ClutterStage *stage = NULL;
       ClutterActor *parent;
+      char *pseudo_class, *direction_pseudo_class;
 
       parent = clutter_actor_get_parent (CLUTTER_ACTOR (widget));
       while (parent != NULL)
@@ -542,24 +576,66 @@ st_widget_get_theme_node (StWidget *widget)
 
       if (stage == NULL)
         {
-          g_error ("st_widget_get_theme_node called on a widget not in a stage");
+          g_error ("st_widget_get_theme_node called on the widget %s which is not in the stage.",
+                    st_describe_actor (CLUTTER_ACTOR (widget)));
         }
 
       if (parent_node == NULL)
         parent_node = get_root_theme_node (CLUTTER_STAGE (stage));
 
+      /* Always append a "magic" pseudo class indicating the text
+       * direction, to allow to adapt the CSS when necessary without
+       * requiring separate style sheets.
+       */
+      if (st_widget_get_direction (widget) == ST_TEXT_DIRECTION_RTL)
+        direction_pseudo_class = "rtl";
+      else
+        direction_pseudo_class = "ltr";
+
+      if (priv->pseudo_class)
+        pseudo_class = g_strconcat(priv->pseudo_class, " ",
+                                   direction_pseudo_class, NULL);
+      else
+        pseudo_class = direction_pseudo_class;
+
       priv->theme_node = st_theme_node_new (st_theme_context_get_for_stage (stage),
                                             parent_node, priv->theme,
                                             G_OBJECT_TYPE (widget),
                                             clutter_actor_get_name (CLUTTER_ACTOR (widget)),
                                             priv->style_class,
-                                            priv->pseudo_class,
+                                            pseudo_class,
                                             priv->inline_style);
+
+      if (pseudo_class != direction_pseudo_class)
+        g_free (pseudo_class);
     }
 
   return priv->theme_node;
 }
 
+/**
+ * st_widget_peek_theme_node:
+ * @widget: a #StWidget
+ *
+ * Returns the theme node for the widget if it has already been
+ * computed, %NULL if the widget hasn't been added to a  stage or the theme
+ * node hasn't been computed. If %NULL is returned, then ::style-changed
+ * will be reliably emitted before the widget is allocated or painted.
+ *
+ * Return value: (transfer none): the theme node for the widget.
+ *   This is owned by the widget. When attributes of the widget
+ *   or the environment that affect the styling change (for example
+ *   the style_class property of the widget), it will be recreated,
+ *   and the ::style-changed signal will be emitted on the widget.
+ */
+StThemeNode *
+st_widget_peek_theme_node (StWidget *widget)
+{
+  StWidgetPrivate *priv = widget->priv;
+
+  return priv->theme_node;
+}
+
 static gboolean
 st_widget_enter (ClutterActor         *actor,
                  ClutterCrossingEvent *event)
@@ -568,7 +644,7 @@ st_widget_enter (ClutterActor         *actor,
 
   if (priv->track_hover)
     {
-      if (_st_actor_contains (actor, event->source))
+      if (clutter_actor_contains (actor, event->source))
         st_widget_set_hover (ST_WIDGET (actor), TRUE);
       else
         {
@@ -594,7 +670,7 @@ st_widget_leave (ClutterActor         *actor,
 
   if (priv->track_hover)
     {
-      if (!_st_actor_contains (actor, event->related))
+      if (!event->related || !clutter_actor_contains (actor, event->related))
         st_widget_set_hover (ST_WIDGET (actor), FALSE);
     }
 
@@ -604,6 +680,50 @@ st_widget_leave (ClutterActor         *actor,
     return FALSE;
 }
 
+static gboolean
+st_widget_motion (ClutterActor       *actor,
+                  ClutterMotionEvent *motion)
+{
+  StWidget *widget = ST_WIDGET (actor);
+  StWidgetPrivate *priv = widget->priv;
+
+  if (priv->has_tooltip)
+    st_widget_start_tooltip_timeout (widget);
+
+  return FALSE;
+}
+
+static void
+st_widget_key_focus_in (ClutterActor *actor)
+{
+  StWidget *widget = ST_WIDGET (actor);
+
+  st_widget_add_style_pseudo_class (widget, "focus");
+}
+
+static void
+st_widget_key_focus_out (ClutterActor *actor)
+{
+  StWidget *widget = ST_WIDGET (actor);
+
+  st_widget_remove_style_pseudo_class (widget, "focus");
+}
+
+static gboolean
+st_widget_key_press_event (ClutterActor    *actor,
+                           ClutterKeyEvent *event)
+{
+  if (event->keyval == CLUTTER_KEY_Menu ||
+      (event->keyval == CLUTTER_KEY_F10 &&
+       (event->modifier_state & CLUTTER_SHIFT_MASK)))
+    {
+      g_signal_emit (actor, signals[POPUP_MENU], 0);
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static void
 st_widget_hide (ClutterActor *actor)
 {
@@ -611,11 +731,44 @@ st_widget_hide (ClutterActor *actor)
 
   /* hide the tooltip, if there is one */
   if (widget->priv->tooltip)
-    st_tooltip_hide (ST_TOOLTIP (widget->priv->tooltip));
+    clutter_actor_hide (CLUTTER_ACTOR (widget->priv->tooltip));
 
   CLUTTER_ACTOR_CLASS (st_widget_parent_class)->hide (actor);
 }
 
+static gboolean
+st_widget_get_paint_volume (ClutterActor *self, ClutterPaintVolume *volume)
+{
+  ClutterActorBox paint_box, alloc_box;
+  StThemeNode *theme_node;
+  StWidgetPrivate *priv;
+  ClutterVertex origin;
+
+  /* Setting the paint volume does not make sense when we don't have any allocation */
+  if (!clutter_actor_has_allocation (self))
+    return FALSE;
+
+  priv = ST_WIDGET(self)->priv;
+
+  theme_node = st_widget_get_theme_node (ST_WIDGET(self));
+  clutter_actor_get_allocation_box (self, &alloc_box);
+
+  if (priv->transition_animation)
+    st_theme_node_transition_get_paint_box (priv->transition_animation,
+                                            &alloc_box, &paint_box);
+  else
+    st_theme_node_get_paint_box (theme_node, &alloc_box, &paint_box);
+
+  origin.x = paint_box.x1 - alloc_box.x1;
+  origin.y = paint_box.y1 - alloc_box.y1;
+  origin.z = 0.0f;
+
+  clutter_paint_volume_set_origin (volume, &origin);
+  clutter_paint_volume_set_width (volume, paint_box.x2 - paint_box.x1);
+  clutter_paint_volume_set_height (volume, paint_box.y2 - paint_box.y1);
+
+  return TRUE;
+}
 
 
 static void
@@ -636,15 +789,24 @@ st_widget_class_init (StWidgetClass *klass)
   actor_class->get_preferred_height = st_widget_get_preferred_height;
   actor_class->allocate = st_widget_allocate;
   actor_class->paint = st_widget_paint;
+  actor_class->get_paint_volume = st_widget_get_paint_volume;
   actor_class->parent_set = st_widget_parent_set;
   actor_class->map = st_widget_map;
   actor_class->unmap = st_widget_unmap;
 
   actor_class->enter_event = st_widget_enter;
   actor_class->leave_event = st_widget_leave;
+  actor_class->motion_event = st_widget_motion;
+  actor_class->key_focus_in = st_widget_key_focus_in;
+  actor_class->key_focus_out = st_widget_key_focus_out;
+  actor_class->key_press_event = st_widget_key_press_event;
   actor_class->hide = st_widget_hide;
 
+  actor_class->get_accessible = st_widget_get_accessible;
+
   klass->style_changed = st_widget_real_style_changed;
+  klass->navigate_focus = st_widget_real_navigate_focus;
+  klass->get_accessible_type = st_widget_accessible_get_type;
 
   /**
    * StWidget:pseudo-class:
@@ -777,8 +939,23 @@ st_widget_class_init (StWidgetClass *klass)
                                    PROP_HOVER,
                                    pspec);
 
+  /**
+   * StWidget:can-focus:
+   *
+   * Whether or not the widget can be focused via keyboard navigation.
+   */
+  pspec = g_param_spec_boolean ("can-focus",
+                                "Can focus",
+                                "Whether the widget can be focused via keyboard navigation",
+                                FALSE,
+                                ST_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class,
+                                   PROP_CAN_FOCUS,
+                                   pspec);
+
   /**
    * StWidget::style-changed:
+   * @widget: the #StWidget
    *
    * Emitted when the style information that the widget derives from the
    * theme changes
@@ -791,6 +968,22 @@ st_widget_class_init (StWidgetClass *klass)
                   NULL, NULL,
                   _st_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
+
+  /**
+   * StWidget::popup-menu:
+   * @widget: the #StWidget
+   *
+   * Emitted when the user has requested a context menu (eg, via a
+   * keybinding)
+   */
+  signals[POPUP_MENU] =
+    g_signal_new ("popup-menu",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (StWidgetClass, popup_menu),
+                  NULL, NULL,
+                  _st_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
 }
 
 /**
@@ -1144,7 +1337,7 @@ st_widget_remove_style_pseudo_class (StWidget    *actor,
 /**
  * st_widget_set_style:
  * @actor: a #StWidget
- * @style_class: (allow-none): a inline style string, or %NULL
+ * @style: (allow-none): a inline style string, or %NULL
  *
  * Set the inline style string for this widget. The inline style string is an
  * optional ';'-separated list of CSS properties that override the style as
@@ -1152,7 +1345,7 @@ st_widget_remove_style_pseudo_class (StWidget    *actor,
  */
 void
 st_widget_set_style (StWidget  *actor,
-                       const gchar *style)
+                     const gchar *style)
 {
   StWidgetPrivate *priv = actor->priv;
 
@@ -1222,6 +1415,7 @@ st_widget_recompute_style (StWidget    *widget,
 {
   StThemeNode *new_theme_node = st_widget_get_theme_node (widget);
   int transition_duration;
+  gboolean paint_equal;
 
   if (!old_theme_node ||
       !st_theme_node_geometry_equal (old_theme_node, new_theme_node))
@@ -1229,6 +1423,11 @@ st_widget_recompute_style (StWidget    *widget,
 
   transition_duration = st_theme_node_get_transition_duration (new_theme_node);
 
+  paint_equal = old_theme_node && st_theme_node_paint_equal (old_theme_node, new_theme_node);
+
+  if (paint_equal)
+    st_theme_node_copy_cached_paint_state (new_theme_node, old_theme_node);
+
   if (transition_duration > 0)
     {
       if (widget->priv->transition_animation != NULL)
@@ -1236,8 +1435,14 @@ st_widget_recompute_style (StWidget    *widget,
           st_theme_node_transition_update (widget->priv->transition_animation,
                                            new_theme_node);
         }
-      else if (old_theme_node)
+      else if (old_theme_node && !paint_equal)
         {
+          /* Since our transitions are only of the painting done by StThemeNode, we
+           * only want to start a transition when what is painted changes; if
+           * other visual aspects like the foreground color of a label change,
+           * we can't animate that anyways.
+           */
+
           widget->priv->transition_animation =
             st_theme_node_transition_new (old_theme_node,
                                           new_theme_node,
@@ -1306,8 +1511,38 @@ st_widget_get_direction (StWidget *self)
 void
 st_widget_set_direction (StWidget *self, StTextDirection dir)
 {
+  StTextDirection old_direction;
+
   g_return_if_fail (ST_IS_WIDGET (self));
+
+  old_direction = st_widget_get_direction (self);
   self->priv->direction = dir;
+
+  if (old_direction != st_widget_get_direction (self))
+    st_widget_style_changed (self);
+}
+
+static void
+st_widget_ensure_tooltip_parented (StWidget *widget, ClutterStage *stage)
+{
+  StWidgetPrivate *priv;
+  ClutterContainer *ui_root;
+  ClutterActor *tooltip, *parent;
+
+  priv = widget->priv;
+
+  ui_root = st_get_ui_root (stage);
+
+  tooltip = CLUTTER_ACTOR (priv->tooltip);
+  parent = clutter_actor_get_parent (tooltip);
+
+  if (G_UNLIKELY (parent != CLUTTER_ACTOR (ui_root)))
+    {
+      if (parent)
+        clutter_container_remove_actor (CLUTTER_CONTAINER (parent), tooltip);
+
+      clutter_container_add_actor (ui_root, tooltip);
+    }
 }
 
 /**
@@ -1327,6 +1562,7 @@ st_widget_set_has_tooltip (StWidget *widget,
                            gboolean  has_tooltip)
 {
   StWidgetPrivate *priv;
+  ClutterActor *stage;
 
   g_return_if_fail (ST_IS_WIDGET (widget));
 
@@ -1342,15 +1578,25 @@ st_widget_set_has_tooltip (StWidget *widget,
       if (!priv->tooltip)
         {
           priv->tooltip = g_object_new (ST_TYPE_TOOLTIP, NULL);
-          clutter_actor_set_parent ((ClutterActor *) priv->tooltip,
-                                    (ClutterActor *) widget);
+          g_object_ref_sink (priv->tooltip);
+
+          stage = clutter_actor_get_stage (CLUTTER_ACTOR (widget));
+          if (stage != NULL)
+            st_widget_ensure_tooltip_parented (widget, CLUTTER_STAGE (stage));
         }
     }
   else
     {
+      if (priv->tooltip_timeout_id)
+        {
+          g_source_remove (priv->tooltip_timeout_id);
+          priv->tooltip_timeout_id = 0;
+        }
+
       if (priv->tooltip)
         {
-          clutter_actor_unparent (CLUTTER_ACTOR (priv->tooltip));
+          clutter_actor_destroy (CLUTTER_ACTOR (priv->tooltip));
+          g_object_unref (priv->tooltip);
           priv->tooltip = NULL;
         }
     }
@@ -1395,9 +1641,10 @@ st_widget_set_tooltip_text (StWidget    *widget,
   if (text == NULL)
     st_widget_set_has_tooltip (widget, FALSE);
   else
-    st_widget_set_has_tooltip (widget, TRUE);
-
-  st_tooltip_set_label (priv->tooltip, text);
+    {
+      st_widget_set_has_tooltip (widget, TRUE);
+      st_tooltip_set_label (priv->tooltip, text);
+    }
 }
 
 /**
@@ -1426,34 +1673,33 @@ st_widget_get_tooltip_text (StWidget *widget)
  * st_widget_show_tooltip:
  * @widget: A #StWidget
  *
- * Show the tooltip for @widget
+ * Force the tooltip for @widget to be shown
  *
  */
 void
 st_widget_show_tooltip (StWidget *widget)
 {
-  gfloat x, y, width, height;
-  ClutterGeometry area;
-
   g_return_if_fail (ST_IS_WIDGET (widget));
 
-  /* XXX not necceary, but first allocate transform is wrong */
+  widget->priv->show_tooltip = TRUE;
+  if (CLUTTER_ACTOR_IS_MAPPED (widget))
+    st_widget_do_show_tooltip (widget);
+}
 
-  clutter_actor_get_transformed_position ((ClutterActor*) widget,
-                                          &x, &y);
-
-  clutter_actor_get_size ((ClutterActor*) widget, &width, &height);
-
-  area.x = x;
-  area.y = y;
-  area.width = width;
-  area.height = height;
+static void
+st_widget_do_show_tooltip (StWidget *widget)
+{
+  ClutterActor *stage, *tooltip;
 
+  stage = clutter_actor_get_stage (CLUTTER_ACTOR (widget));
+  g_return_if_fail (stage != NULL);
 
   if (widget->priv->tooltip)
     {
-      st_tooltip_set_tip_area (widget->priv->tooltip, &area);
-      st_tooltip_show (widget->priv->tooltip);
+      tooltip = CLUTTER_ACTOR (widget->priv->tooltip);
+      st_widget_ensure_tooltip_parented (widget, CLUTTER_STAGE (stage));
+      clutter_actor_raise (tooltip, NULL);
+      clutter_actor_show_all (tooltip);
     }
 }
 
@@ -1469,8 +1715,16 @@ st_widget_hide_tooltip (StWidget *widget)
 {
   g_return_if_fail (ST_IS_WIDGET (widget));
 
+  widget->priv->show_tooltip = FALSE;
+  if (CLUTTER_ACTOR_IS_MAPPED (widget))
+    st_widget_do_hide_tooltip (widget);
+}
+
+static void
+st_widget_do_hide_tooltip (StWidget *widget)
+{
   if (widget->priv->tooltip)
-    st_tooltip_hide (widget->priv->tooltip);
+    clutter_actor_hide (CLUTTER_ACTOR (widget->priv->tooltip));
 }
 
 /**
@@ -1528,6 +1782,28 @@ st_widget_get_track_hover (StWidget *widget)
   return widget->priv->track_hover;
 }
 
+static gboolean
+tooltip_timeout (gpointer data)
+{
+  st_widget_show_tooltip (data);
+
+  return FALSE;
+}
+
+static void
+st_widget_start_tooltip_timeout (StWidget *widget)
+{
+  StWidgetPrivate *priv = widget->priv;
+  GtkSettings *settings = gtk_settings_get_default ();
+  guint timeout;
+
+  if (priv->tooltip_timeout_id)
+    g_source_remove (priv->tooltip_timeout_id);
+
+  g_object_get (settings, "gtk-tooltip-timeout", &timeout, NULL);
+  priv->tooltip_timeout_id = g_timeout_add (timeout, tooltip_timeout, widget);
+}
+
 /**
  * st_widget_set_hover:
  * @widget: A #StWidget
@@ -1558,13 +1834,21 @@ st_widget_set_hover (StWidget *widget,
         {
           st_widget_add_style_pseudo_class (widget, "hover");
           if (priv->has_tooltip)
-            st_widget_show_tooltip (widget);
+            st_widget_start_tooltip_timeout (widget);
         }
       else
         {
           st_widget_remove_style_pseudo_class (widget, "hover");
           if (priv->has_tooltip)
-            st_widget_hide_tooltip (widget);
+            {
+              if (priv->tooltip_timeout_id)
+                {
+                  g_source_remove (priv->tooltip_timeout_id);
+                  priv->tooltip_timeout_id = 0;
+                }
+
+              st_widget_hide_tooltip (widget);
+            }
         }
       g_object_notify (G_OBJECT (widget), "hover");
     }
@@ -1589,7 +1873,10 @@ st_widget_sync_hover (StWidget *widget)
   pointer = clutter_device_manager_get_core_device (device_manager,
                                                     CLUTTER_POINTER_DEVICE);
   pointer_actor = clutter_input_device_get_pointer_actor (pointer);
-  st_widget_set_hover (widget, _st_actor_contains (CLUTTER_ACTOR (widget), pointer_actor));
+  if (pointer_actor)
+    st_widget_set_hover (widget, clutter_actor_contains (CLUTTER_ACTOR (widget), pointer_actor));
+  else
+    st_widget_set_hover (widget, FALSE);
 }
 
 /**
@@ -1609,6 +1896,113 @@ st_widget_get_hover (StWidget *widget)
   return widget->priv->hover;
 }
 
+/**
+ * st_widget_set_can_focus:
+ * @widget: A #StWidget
+ * @can_focus: %TRUE if the widget can receive keyboard focus
+ *   via keyboard navigation
+ *
+ * Marks @widget as being able to receive keyboard focus via
+ * keyboard navigation.
+ */
+void
+st_widget_set_can_focus (StWidget *widget,
+                         gboolean  can_focus)
+{
+  StWidgetPrivate *priv;
+
+  g_return_if_fail (ST_IS_WIDGET (widget));
+
+  priv = widget->priv;
+
+  if (priv->can_focus != can_focus)
+    {
+      priv->can_focus = can_focus;
+      g_object_notify (G_OBJECT (widget), "can-focus");
+    }
+}
+
+/**
+ * st_widget_get_can_focus:
+ * @widget: A #StWidget
+ *
+ * Returns the current value of the can-focus property. See
+ * st_widget_set_can_focus() for more information.
+ *
+ * Returns: current value of can-focus on @widget
+ */
+gboolean
+st_widget_get_can_focus (StWidget *widget)
+{
+  g_return_val_if_fail (ST_IS_WIDGET (widget), FALSE);
+
+  return widget->priv->can_focus;
+}
+
+static gboolean
+st_widget_real_navigate_focus (StWidget         *widget,
+                               ClutterActor     *from,
+                               GtkDirectionType  direction)
+{
+  if (widget->priv->can_focus &&
+      CLUTTER_ACTOR (widget) != from)
+    {
+      clutter_actor_grab_key_focus (CLUTTER_ACTOR (widget));
+      return TRUE;
+    }
+  return FALSE;
+}
+
+/**
+ * st_widget_navigate_focus:
+ * @widget: the "top level" container
+ * @from: (allow-none): the actor that the focus is coming from
+ * @direction: the direction focus is moving in
+ * @wrap_around: whether focus should wrap around
+ *
+ * Tries to update the keyboard focus within @widget in response to a
+ * keyboard event.
+ *
+ * If @from is a descendant of @widget, this attempts to move the
+ * keyboard focus to the next descendant of @widget (in the order
+ * implied by @direction) that has the #StWidget:can-focus property
+ * set. If @from is %NULL, or outside of @widget, this attempts to
+ * focus either @widget itself, or its first descendant in the order
+ * implied by @direction.
+ *
+ * If a container type is marked #StWidget:can-focus, the expected
+ * behavior is that it will only take up a single slot on the focus
+ * chain as a whole, rather than allowing navigation between its child
+ * actors (or having a distinction between itself being focused and
+ * one of its children being focused).
+ *
+ * Some widget classes might have slightly different behavior from the
+ * above, where that would make more sense.
+ *
+ * If @wrap_around is %TRUE and @from is a child of @widget, but the
+ * widget has no further children that can accept the focus in the
+ * given direction, then st_widget_navigate_focus() will try a second
+ * time, using a %NULL @from, which should cause it to reset the focus
+ * to the first available widget in the given direction.
+ *
+ * Return value: %TRUE if clutter_actor_grab_key_focus() has been
+ * called on an actor. %FALSE if not.
+ */
+gboolean
+st_widget_navigate_focus (StWidget         *widget,
+                          ClutterActor     *from,
+                          GtkDirectionType  direction,
+                          gboolean          wrap_around)
+{
+  g_return_val_if_fail (ST_IS_WIDGET (widget), FALSE);
+
+  if (ST_WIDGET_GET_CLASS (widget)->navigate_focus (widget, from, direction))
+    return TRUE;
+  if (wrap_around && from && clutter_actor_contains (CLUTTER_ACTOR (widget), from))
+    return ST_WIDGET_GET_CLASS (widget)->navigate_focus (widget, NULL, direction);
+  return FALSE;
+}
+
 static gboolean
 append_actor_text (GString      *desc,
                    ClutterActor *actor)
@@ -1730,3 +2124,261 @@ st_get_slow_down_factor ()
 {
   return st_slow_down_factor;
 }
+
+/******************************************************************************/
+/*************************** ACCESSIBILITY SUPPORT ****************************/
+/******************************************************************************/
+
+/* GObject */
+
+static void st_widget_accessible_class_init (StWidgetAccessibleClass *klass);
+static void st_widget_accessible_init       (StWidgetAccessible *widget);
+
+/* AtkObject */
+static AtkStateSet *st_widget_accessible_ref_state_set (AtkObject *obj);
+static void         st_widget_accessible_initialize    (AtkObject *obj,
+                                                        gpointer   data);
+
+/* Private methods */
+static void on_pseudo_class_notify (GObject    *gobject,
+                                    GParamSpec *pspec,
+                                    gpointer    data);
+static void on_can_focus_notify    (GObject    *gobject,
+                                    GParamSpec *pspec,
+                                    gpointer    data);
+static void check_selected         (StWidgetAccessible *self,
+                                    StWidget *widget);
+
+G_DEFINE_TYPE (StWidgetAccessible, st_widget_accessible, CALLY_TYPE_ACTOR)
+
+#define ST_WIDGET_ACCESSIBLE_GET_PRIVATE(obj) \
+  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), ST_TYPE_WIDGET_ACCESSIBLE, \
+                                StWidgetAccessiblePrivate))
+
+struct _StWidgetAccessiblePrivate
+{
+  /* Cached values (used to avoid extra notifications) */
+  gboolean selected;
+};
+
+
+static AtkObject *
+st_widget_get_accessible (ClutterActor *actor)
+{
+  StWidget *widget = NULL;
+
+  g_return_val_if_fail (ST_IS_WIDGET (actor), NULL);
+
+  widget = ST_WIDGET (actor);
+
+  if (widget->priv->accessible == NULL)
+    {
+      widget->priv->accessible =
+        g_object_new (ST_WIDGET_GET_CLASS (widget)->get_accessible_type (),
+                      NULL);
+
+      atk_object_initialize (widget->priv->accessible, actor);
+    }
+
+  return widget->priv->accessible;
+}
+
+
+static void
+st_widget_accessible_class_init (StWidgetAccessibleClass *klass)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  AtkObjectClass *atk_class = ATK_OBJECT_CLASS (klass);
+
+  atk_class->ref_state_set = st_widget_accessible_ref_state_set;
+  atk_class->initialize = st_widget_accessible_initialize;
+
+  g_type_class_add_private (gobject_class, sizeof (StWidgetAccessiblePrivate));
+}
+
+static void
+st_widget_accessible_init (StWidgetAccessible *self)
+{
+  StWidgetAccessiblePrivate *priv = ST_WIDGET_ACCESSIBLE_GET_PRIVATE (self);
+
+  self->priv = priv;
+}
+
+static void
+st_widget_accessible_initialize (AtkObject *obj,
+                                 gpointer   data)
+{
+  ATK_OBJECT_CLASS (st_widget_accessible_parent_class)->initialize (obj, data);
+
+  g_signal_connect (data, "notify::pseudo-class",
+                    G_CALLBACK (on_pseudo_class_notify),
+                    obj);
+
+  g_signal_connect (data, "notify::can-focus",
+                    G_CALLBACK (on_can_focus_notify),
+                    obj);
+
+  /* Check the cached selected state and notify the first selection.
+   * Ie: it is required to ensure a first notification when Alt+Tab
+   * popup appears
+   */
+  check_selected (ST_WIDGET_ACCESSIBLE (obj), ST_WIDGET (data));
+}
+
+static AtkStateSet *
+st_widget_accessible_ref_state_set (AtkObject *obj)
+{
+  AtkStateSet *result = NULL;
+  ClutterActor *actor = NULL;
+  StWidget *widget = NULL;
+  StWidgetAccessible *self = NULL;
+
+  result = ATK_OBJECT_CLASS (st_widget_accessible_parent_class)->ref_state_set (obj);
+
+  actor = CLUTTER_ACTOR (atk_gobject_accessible_get_object (ATK_GOBJECT_ACCESSIBLE (obj)));
+
+  if (actor == NULL) /* State is defunct */
+    return result;
+
+  widget = ST_WIDGET (actor);
+  self = ST_WIDGET_ACCESSIBLE (obj);
+
+  /* priv->selected should be properly updated on the
+   * ATK_STATE_SELECTED notification callbacks
+   */
+  if (self->priv->selected)
+    atk_state_set_add_state (result, ATK_STATE_SELECTED);
+
+  /* On clutter there isn't any tip to know if a actor is focusable or
+   * not, anyone can receive the key_focus. For this reason
+   * cally_actor sets any actor as FOCUSABLE. This is not the case on
+   * St, where we have can_focus. But this means that we need to
+   * remove the state FOCUSABLE if it is not focusable
+   */
+  if (st_widget_get_can_focus (widget))
+    atk_state_set_add_state (result, ATK_STATE_FOCUSABLE);
+  else
+    atk_state_set_remove_state (result, ATK_STATE_FOCUSABLE);
+
+  return result;
+}
+
+static void
+on_pseudo_class_notify (GObject    *gobject,
+                        GParamSpec *pspec,
+                        gpointer    data)
+{
+  check_selected (ST_WIDGET_ACCESSIBLE (data),
+                  ST_WIDGET (gobject));
+}
+
+/*
+ * This method checks if the widget is selected, and notify a atk
+ * state change if required
+ *
+ * In order to decide if there was a selection, we use the current
+ * pseudo-class of the widget searching for "selected", the current
+ * homogeneus way to check if a item is selected (see bug 637830)
+ *
+ * In a ideal world we would have a more standard way to check if the
+ * item is selected or not, like the widget-context (as in the case of
+ * gtktreeview-cells), or something like the property "can-focus". But
+ * for the moment this is enough, and we can update that in the future
+ * if required.
+ */
+static void
+check_selected (StWidgetAccessible *self,
+                StWidget *widget)
+{
+  gboolean found = FALSE;
+
+  found = st_widget_has_style_pseudo_class (widget,
+                                            "selected");
+
+  if (found != self->priv->selected)
+    {
+      self->priv->selected = found;
+      atk_object_notify_state_change (ATK_OBJECT (self),
+                                      ATK_STATE_SELECTED,
+                                      found);
+    }
+}
+
+static void
+on_can_focus_notify (GObject    *gobject,
+                     GParamSpec *pspec,
+                     gpointer    data)
+{
+  gboolean can_focus = st_widget_get_can_focus (ST_WIDGET (gobject));
+
+  atk_object_notify_state_change (ATK_OBJECT (data),
+                                  ATK_STATE_FOCUSABLE, can_focus);
+}
+
+static GQuark
+st_ui_root_quark (void)
+{
+  static GQuark value = 0;
+  if (G_UNLIKELY (value == 0))
+    value = g_quark_from_static_string ("st-ui-root");
+  return value;
+}
+
+static void
+st_ui_root_destroyed (ClutterActor *actor,
+                      ClutterStage *stage)
+{
+  st_set_ui_root (stage, NULL);
+  g_signal_handlers_disconnect_by_func (actor, st_ui_root_destroyed, stage);
+}
+
+/**
+ * st_get_ui_root:
+ * @stage: a #ClutterStage
+ * @container: (allow-none): the new UI root
+ *
+ * Sets a #ClutterContainer to be the parent of all UI in the program.
+ * This container is used when St needs to add new content outside the
+ * widget hierarchy, for example, when it shows a tooltip over a widget.
+ */
+void
+st_set_ui_root (ClutterStage     *stage,
+                ClutterContainer *container)
+{
+  ClutterContainer *previous;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+  g_return_if_fail (CLUTTER_IS_CONTAINER (container));
+
+  previous = st_get_ui_root (stage);
+  if (previous)
+    g_signal_handlers_disconnect_by_func (container, st_ui_root_destroyed, stage);
+
+  if (container)
+    {
+      g_signal_connect (container, "destroy", G_CALLBACK (st_ui_root_destroyed), stage);
+      g_object_set_qdata_full (G_OBJECT (stage), st_ui_root_quark (), g_object_ref (container), g_object_unref);
+    }
+}
+
+/**
+ * st_get_ui_root:
+ * @stage: a #ClutterStage
+ *
+ * Returns: (transfer none): the container which should be the parent of all user interface,
+ *   which can be set with st_set_ui_root(). If not set, returns @stage
+ */
+ClutterContainer *
+st_get_ui_root (ClutterStage *stage)
+{
+  ClutterContainer *root;
+
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), NULL);
+
+  root = g_object_get_qdata (G_OBJECT (stage), st_ui_root_quark ());
+
+  if (root != NULL)
+    return root;
+  else
+    return CLUTTER_CONTAINER (stage);
+}
